/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2015.01.02.
*   Chip         -  Atmel ATmega8 & HD44780
*   Compiler     -  avr-gcc (WinAVR)
*
*   Sajat kpri() - nagyjabol printf
*   
*******************************************************************************/

#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdarg.h>




char * kpri( const char *, ...);                 // sajat printf
char * kpri_u2s( unsigned int, unsigned int, unsigned char, unsigned char);
                                                 // szam atalakitasa string-re
char * kpri_pad( char *, char *, unsigned char, unsigned char, unsigned char);
                                                 // string levagasa, feltoltese, athelyezese
 



/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2014.09.06.
*
*   Sajat kpri() - nagyjabol a printf-bol amire szuksegem volt. A kimenetet
*   string-be irja. A fuggveny nem nagyon ellenorzi az ertekeket, ezert mas
*   kombinaciok is beallithatok, de talan ezeknek van ertelme. Szinten nem
*   figyel a buffer tulcsordulasra. Hoszabb kiiras eseten noveljuk meg a
*   buffer-t. Nekem LCD-re csak 16 betu fer, USART-ra gondolva 40-re allitottam.
*   Rovidebb hossz beallitasa eseten a szamok elejet levagja!
*   Helytakarekossagbol a formatum string a program memoriaba (PSTR) kerul.  
*
*   %b    - binaris szam, fix 8 betu, balrol 0-val feltoltve, unsigned char
*
*   %c    - 1 betu kiirasa
*   %5c   - 5 azonos betu kiirasa
*
*   %d    - signed int decimalis kiirasa, amilyen hosszu, elojellel egyutt
*   %5d   - 5 betu hosszu, elojellel egyutt, balrol szokozzel feltoltve 
*   %05d  - 5 betu hosszu, elojellel egyutt, balrol 0-val feltoltve 
*   %-5d  - 5 betu hosszu, elojellel egyutt jobbrol szokozzel feltoltve 
*   %5.2d - 5 betu hosszu, tizedes ponttal s elojellel egyutt, jobb oldalt ket tizedes jegy
*    
*   %s    - string beszurasa
*   %5s   - string utolso 5 betujenek kiirasa, ha rovidebb, balrol szokozzel feltoltve
*   %-5s  - string utolso 5 betujenek kiirasa, ha rovidebb, jobbrol szokozzel feltoltve
*
*   %u    - mint %d, de unsigned int
*   
*   %x    - unsigned int hexadecimalis kiirasa
*   %2x   - hexa szam ket utolso betuje
*   
*******************************************************************************/
char * kpri( const char * frmt, ...)
{
   va_list ap;
   static char buffer[ 32];
   const char * f;                               // mutato formatum stringre
   char * t;                                     // mutato a bufferre
   int i;                                        // int szam
   unsigned int u;                               // unsigned int szam
   char * s;                                     // mutato string-re
   char * n;                                     // mutato szam string-re
   unsigned char jelzo= 0;                       // 0 - alapeset, egy beturol van szo 
                                                 // 1 - % utan
                                                 // 2 - . utan
   unsigned char c;                              // egy betu tarolasara
   unsigned char balra= 0;                       // hova legyen utkoztetve a string
   unsigned char hossz= 0;                       // string hossza
   unsigned char elojel_betu= 0;                 // elojel
   unsigned char tizedesjegy= 0;                 // tizedes jegyek szama
   unsigned char feltolto_betu= ' ';             // betu, string feltoltesehez
   va_start( ap, frmt);
   f= frmt;                                      // formatum string
   t= buffer;                                    // buffer eleje
   while( (c= pgm_read_byte( f++)))              // formatum string beolvasasa betunkent
   {
      if( ( jelzo == 0) && ( c != '%')) *t++= c; // kiirando betu
      else
      {
         switch( c)
         {
            case '%': 
               if( jelzo)                        // masodik %
               {
                  *t++= c;
                  jelzo= 0;
               }
               else                              // elso % eseten
               {
                  jelzo= 1;                      // jelzo beallitasa
                  balra= 0;                      // alap ertekek beallitasa
                  hossz= 0;
                  elojel_betu= 0;
                  tizedesjegy= 0;
                  feltolto_betu= ' ';
               }
               break;
            case '-': 
               if( jelzo == 1) balra= 1;         // balra kell utkoztetni
               break;
            case '.': 
               if( jelzo == 1) jelzo= 2;         // tizedespont utani ertek fog kovetkezni
               if( hossz) hossz++;               // tizedespont hosszaval valomegnoveles
               break;
            case '0': 
            case '1': 
            case '2': 
            case '3': 
            case '4': 
            case '5': 
            case '6': 
            case '7': 
            case '8': 
            case '9': 
               if( jelzo == 1)                   // ha % utan vagyunk
               {                                 // kezdo 0, nullaval valo feltoltest jelol
                  if( ( c == '0') && (hossz == 0)) feltolto_betu= '0';
                  else
                  {
                     if( hossz) hossz*= 10;      // tobb szamjegy eseten
                     hossz+= c- '0';             // hosszusag beallitasa
                  }
               } 
               if( jelzo == 2)                   // ha tizedespont utan vagyunk
               {  
                  if( tizedesjegy) tizedesjegy*= 10;
                  tizedesjegy+= c- '0';          // tizedesjegyek beallitasa
               }
               break; 
            case 'b':                            // binaris kiiras - 00000000..11111111
               u= 0x00FF & va_arg( ap, unsigned int);
               n= kpri_u2s( u, 2, 0, 0);
               t= kpri_pad( n, t, 8, 1, '0');
               jelzo= 0;                         // visszaalitas alapra
               break;
            case 'c':                            // 1..hossz char kiirasa
               c= (char)va_arg( ap, unsigned int); 
               if( hossz == 0) hossz= 1;
               while( hossz--) *t++= c; 
               jelzo= 0;
               break;
            case 'd':                            // signed int decimalis kiirasa - +/- 0..32767
               i= va_arg( ap, int);
               if( i < 0)                        // negativ szam eseten
               {
                  i= -i;
                  elojel_betu= '-';
               }   
               n= kpri_u2s( (unsigned int)i, 10, elojel_betu, tizedesjegy);
               t= kpri_pad( n, t, hossz, balra, feltolto_betu);
               jelzo= 0;
               break;
            case 'u':                            // unsigned int decimalis kiirasa - 0..65535
               u= va_arg( ap, unsigned int);
               n= kpri_u2s( u, 10, 0, tizedesjegy);
               t= kpri_pad( n, t, hossz, balra, feltolto_betu);
               jelzo= 0;
               break;
            case 'x':                            // unsigned int hexadecimalis kiirasa - 0..FFFF
               u= va_arg( ap, unsigned int);
               n= kpri_u2s( u, 16, 0, tizedesjegy);
               t= kpri_pad( n, t, hossz, balra, feltolto_betu);
               jelzo= 0;
               break;
            case 's':                            // string beszurasa
               s= va_arg( ap, char *);
               t= kpri_pad( s, t, hossz, balra, feltolto_betu);
               jelzo= 0;
               break;
         }
      }
   }
   *t= '\0';                                     // string vege
   va_end( ap);
   return buffer;  
}




/******************************************************************************* 
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2014.09.06.
*
*   string levagasa, feltoltese bal/jobb oldalt es athelyezese
*
*   honnan - hol talalhato a string
*   hova - hova kell atmasolni a string-et
*   hossz - a string hossza
*   balra - balra vagy jobbra kell utkoztetni
*   feltolto_betu - ha rovidebb mint hossz, az ures betuk helyere irando betu 
*
*******************************************************************************/ 
char * kpri_pad(
   char * honnan,
   char * hova,
   unsigned char hossz,
   unsigned char balra,
   unsigned char feltolto_betu
)
{
   char * p;
   unsigned char l= 0;
   p= honnan;
   while( *p++) l++;                                  // hossz kiszamolasa - strlen()
   if( hossz == 0) hossz= l;                          // hossz == 0 eseten a teljes hossz 
   while( l > hossz )                                 // vagas, ha a string hoszabb mint a megadott hossz
   {
      l--;
      honnan++; 
   }
   l= hossz- l;                                       // hany feltolto betu kell
   if( !balra) while( l--) *hova++= feltolto_betu;    // feltoltes string-tol balra
   while( *honnan) *hova++= *honnan++;                // string masolasa 
   if( balra) while( l--) *hova++= feltolto_betu;     // feltoltes string-tol jobbra
   return hova;                                       // kovetkezo char helye
}




/******************************************************************************* 
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2014.09.06.
*
*   unsigned int szam konvertalasa a kivant alapu szamrendszerbeli string-re
*
*   elojel_betu - ha 0, nincs elojel
*   tizedesjegy - tizedespont utani szamjegyek szama, ha 0, nincs tizedespont
*
*******************************************************************************/ 
char * kpri_u2s(
   unsigned int szam,
   unsigned int alap,
   unsigned char elojel_betu,
   unsigned char tizedesjegy
)                                                // szam -> string
{
   static char buffer[10];
   unsigned char n= 0;                                      // betuk szama
   char * p; 
   p= &buffer[ sizeof( buffer)- 1];
   *p= '\0';                                     // string vege, hatulrol elore
   if( tizedesjegy) tizedesjegy++;
   do
   {
      n++;
      p--;          
      if( n == tizedesjegy) *p= '.';             // ha tizedespont
      else
      {
         *p= szam% alap;                         // maradek a szamjegy
         if( *p > 9) *p+= 55;                    // 9-nel nagyobb, akkor ABC..
         else *p+= '0';                          // 9-ig szamjegy
         szam/= alap;
      }
   } while( szam != 0);
   if( elojel_betu) *--p= elojel_betu;           // ha van elojel
   return p;                                     // mutato a stringre
}