/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2015.01.02.
*   Chip         -  Atmel ATmega8 & HD44780
*   Compiler     -  avr-gcc (WinAVR)
*
*   LCD kezelese (Hitachi)
*   
********************************************************************************
*   Alapertelmezett kiosztas:
*
*   LCD:
*   14   LCD D7       - AVR PD7  - 13
*   13   LCD D6       - AVR PD6  - 12
*   12   LCD D5       - AVR PD5  - 11
*   11   LCD D4       - AVR PD4  -  6
*   10   LCD D3
*    9   LCD D2
*    8   LCD D1
*    7   LCD D0
*    6   LCD E        - AVR PB7  - 10
*    5   LCD RW       - GND
*    4   LCD RS       - AVR PB6  -  9
*    3   LCD KONTR    - 10 kOhm trim.
*    2   LCD VDD      - +5V
*    1   LCD VSS      - GND
*     
*
*   A foprogramban ehhez a kiosztashoz szukseges definiciok:
*
*
// LCD_E - PB7
#define LCD_E_ENABLE         PB7_OUT
#define LCD_E_0              BC( PORTB, 7)
#define LCD_E_1              BS( PORTB, 7)

// LCD_RS - PB6
#define LCD_RS_ENABLE        PB6_OUT
#define LCD_RS_UTASITAS      BC( PORTB, 6)
#define LCD_RS_ADAT          BS( PORTB, 6)

// LCD adat port - PORTD felso 4 bit - KBD sorok is
#define LCD_PORT             PORTD
#define LCD_PORT_ENABLE      DDRD|= 0B11110000
#define LCD_PORT_DISABLE     DDRD&= 0B00001111
*
*
*******************************************************************************/

#include <util/delay.h>
#include <avr/pgmspace.h>




void lcd_init4( void);                           // LCD inicializalasa 4 bitre
void lcd_putc( unsigned char);                   // betu kiirasa
void lcd_putcmd( unsigned char);                 // parancskod kiadasa
void lcd_yx( unsigned char, unsigned char);      // kurzor pozicioja 0..3/0..15
void lcd_cls( void);                             // kepernyo torles
void lcd_puts( const char *);                    // string kiirasa
void lcdre( char *);                             // string kiiras LCD-re




void lcd_init4( void)                            // LCD inicializalasa 4 bitre
{
   LCD_PORT_ENABLE;
   LCD_E_ENABLE;
   LCD_RS_ENABLE;
   _delay_ms( 15);
   LCD_RS_UTASITAS;
   LCD_PORT= 0B00100000;                         // 4 bit interface
   LCD_E_1;
   LCD_E_0;
   _delay_ms( 5);
   LCD_E_1;
   LCD_E_0;
   _delay_us( 120);
   lcd_putc( 0B00101000);                        // 4 bit interface, 2 sor, 5x8 pontos betu
   lcd_putc( 0B00101000);                        // 2x kell kiadni, vagy LCD D3-t VDD-re kell kotni
   lcd_putc( 0B00001100);                        // kijelzes be, cursor ki
   lcd_putc( 0B00000110);                        // kiiras jobbra
   LCD_RS_ADAT;
}




void lcd_putcmd( unsigned char cmd)              // parancskod kiadasa
{
   LCD_RS_UTASITAS;
   lcd_putc( cmd);
   LCD_RS_ADAT;
}




void lcd_putc( unsigned char c)                  // egy betu kiiras
{
   LCD_E_1;                                      // felso 4 bit
   LCD_PORT= ( LCD_PORT & 0x0F) | ( 0xF0 & c);
   LCD_E_0;
   LCD_E_1;                                      // also 4 bit
   LCD_PORT= ( LCD_PORT & 0x0F) | ( 0xF0 & c << 4);
   LCD_E_0;
   _delay_us( 37);                               // var 37 usec  
}




void lcd_yx( unsigned char sor, unsigned char betu)
                                                 //  kurzor pozicionalasa 0..3/0..15
{
   unsigned char cim= 0x80;                      // parancs kodja
   if (sor & 0B00000001) cim+= 64;               // 1. es 3. sor
   if (sor & 0B00000010) cim+= 20;               // 2. es 3. sor
   cim+= betu & 0x0F; 
   lcd_putcmd( cim);
}




void lcd_cls( void)                              // kepernyo torles
{
   lcd_putcmd( 0x01);
   _delay_ms( 2);
}




void lcd_puts( const char *s)                    // string kiiras
{
   register unsigned char c;
   while ( ( c= pgm_read_byte( s++))) lcd_putc( c);
}




void lcdre( char *s)                             // string kiiras LCD-re
{
   while( *s) lcd_putc( *s++);
}
